@@ -9,35 +9,39 @@
 
 #include "osl/record/usi.h"
 #include "osl/record/opening/openingBook.h"
-#include "osl/record/csaRecord.h"
 #include "osl/record/csaIOError.h"
 #include "osl/state/numEffectState.h"
+#include "osl/game_playing/usiState.h"
 #include "osl/game_playing/speculativeSearchPlayer.h"
 #include "osl/game_playing/alphaBetaPlayer.h"
 #include "osl/game_playing/gameState.h"
 #include "osl/game_playing/weightTracer.h"
 #include "osl/game_playing/bookPlayer.h"
 #include "osl/game_playing/csaLogger.h"
+#include "osl/game_playing/usiResponse.h"
+#include "osl/search/usiReporter.h"
 #include "osl/checkmate/dfpn.h"
 #include "osl/checkmate/dfpnParallel.h"
 #include "osl/eval/ml/openMidEndingEval.h"
 #include "osl/eval/progressEval.h"
 #include "osl/rating/featureSet.h"
-#include "osl/misc/pointerQueue.h"
-#include "osl/misc/ctime.h"
-#include "osl/misc/ncores.h"
 #include "osl/progress/ml/newProgress.h"
 #include "osl/move_generator/legalMoves.h"
 #include "osl/move_probability/featureSet.h"
-#include "osl/rating/ratingEnv.h"
 #include "osl/oslConfig.h"
 #include "osl/sennichite.h"
 #include "osl/enter_king/enterKing.h"
+#include "osl/misc/pointerQueue.h"
+#include "osl/misc/ctime.h"
+#include "osl/misc/ncores.h"
 #include <boost/scoped_ptr.hpp>
 #include <boost/thread/thread.hpp>
 #include <boost/thread/mutex.hpp>
 #include <boost/program_options.hpp>
 #include <boost/foreach.hpp>
+#include <boost/algorithm/string/split.hpp>
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/asio.hpp>
 namespace po = boost::program_options;
 #include <string>
 #include <sstream>
@@ -71,6 +75,11 @@ int book_width_root = 16, book_width = 10;
 #endif
 uint64_t node_count_hard_limit = 0;
 bool new_move_probability = true;
+std::string udp_log = "";
+boost::asio::io_service io_service;
+using boost::asio::ip::udp;
+boost::scoped_ptr<udp::socket> udp_socket;
+boost::scoped_ptr<udp::endpoint> udp_endpoint;
 
 bool is_verbose()
 {
@@ -85,29 +94,6 @@ volatile bool stop_checkmate = false;
 boost::mutex go_stop_mutex;
 boost::condition go_stop_condition;
 
-struct UsiState
-{
-  SimpleState initial_state;
-  vector<Move> moves;
-  volatile bool aborted;
-  UsiState() : aborted(false) {}
-
-  bool isSuccessorOf(const UsiState& parent)
-  {
-    return ! aborted && ! parent.aborted
-      && initial_state == parent.initial_state
-      && moves.size() == parent.moves.size()+1
-      && std::equal(parent.moves.begin(), parent.moves.end(), moves.begin());
-  }
-  const NumEffectState currentState() const
-  {
-    NumEffectState state(initial_state);
-    BOOST_FOREACH(Move m, moves)
-      state.makeMove(m);
-    return state;
-  }
-};
-
 boost::scoped_ptr<std::ofstream> csa_stream;
 boost::scoped_ptr<game_playing::CsaLogger> csa_logger;
 struct StopWatch
@@ -133,13 +119,14 @@ class StatefulPlayer
   boost::scoped_ptr<game_playing::ComputerPlayer> player;
   CArray<int,2> initial_time;
   Player my_turn;
+  std::ostream& os;
 public:
   mutable boost::mutex mutex;
   StatefulPlayer(const UsiState& initial_state,
 		 game_playing::ComputerPlayer *p,
-		 Player turn)
+		 Player turn, std::ostream& o)
     : usi_state(initial_state), state(initial_state.initial_state),
-      player(p), my_turn(turn)
+      player(p), my_turn(turn), os(o)
   {
     initial_time.fill(-1);
     if (csa_logger) {
@@ -210,11 +197,10 @@ public:
     }
     player->setRootIgnoreMoves(0, false);
     ignore_moves.clear();
-    OslConfig::resetRootWindow();
     std::string result = "bestmove " + record::usi::show(ret.move);
     {
       boost::mutex::scoped_lock lk(OslConfig::lock_io);
-      std::cout << result << "\n" << std::flush;
+      os << result << "\n" << std::flush;
     }
     if (is_verbose())
       std::cerr << "sent: " << result << "\n";
@@ -307,7 +293,7 @@ struct ThreadRun
 boost::scoped_ptr<UsiState> input_state;
 boost::scoped_ptr<boost::thread> search_thread;
 
-game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState&);
+game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState&, std::ostream&);
 Player input_turn() 
 {
   Player turn = input_state->initial_state.turn();
@@ -315,7 +301,7 @@ Player input_turn()
     turn = alt(input_state->moves.back().player());
   return turn;
 }
-void do_search()
+void do_search(std::ostream& os)
 {
   if (search_thread && input_state->isSuccessorOf(player->current())) {
     if (debug)
@@ -337,8 +323,8 @@ void do_search()
     csa_stream.reset(new std::ofstream(csa_file.c_str()));
     csa_logger.reset(new game_playing::CsaLogger(*csa_stream));
   } 
-  game_playing::ComputerPlayer *searcher = makePlayer(turn, input_state->initial_state);
-  player.reset(new StatefulPlayer(*input_state, searcher, turn));
+  game_playing::ComputerPlayer *searcher = makePlayer(turn, input_state->initial_state, os);
+  player.reset(new StatefulPlayer(*input_state, searcher, turn, os));
   search_thread.reset(new boost::thread(ThreadRun()));  
 
   boost::mutex::scoped_lock lk(go_stop_mutex);
@@ -388,7 +374,8 @@ struct GoMateThread
     }
   };
   double seconds;
-  explicit GoMateThread(double s) : seconds(s)
+  std::ostream& os;
+  explicit GoMateThread(double s, std::ostream& o) : seconds(s), os(o)
   {
   }
   void operator()();
@@ -401,7 +388,7 @@ void GoMateThread::operator()()
 #if (! defined ALLOW_KING_ABSENCE)
   if (state.kingSquare(state.turn()).isPieceStand()) {
     boost::mutex::scoped_lock lk(OslConfig::lock_io);
-    std::cout << "checkmate notimplemented\n";
+    os << "checkmate notimplemented\n";
     return;
   }
 #endif  
@@ -433,9 +420,9 @@ void GoMateThread::operator()()
     if (i % 4 == 0) {
       double memory = OslConfig::memoryUseRatio();
       boost::mutex::scoped_lock lk(OslConfig::lock_io);
-      std::cout << "info time " << static_cast<int>(elapsed*1000)
-		<< " nodes " << node_count << " nps " << static_cast<int>(node_count/elapsed)
-		<< " hashfull " << static_cast<int>(memory*1000) << "\n";
+      os << "info time " << static_cast<int>(elapsed*1000)
+	 << " nodes " << node_count << " nps " << static_cast<int>(node_count/elapsed)
+	 << " hashfull " << static_cast<int>(memory*1000) << "\n";
     }
     if ((seconds*0.94-elapsed)*1000 < 110 || stop_checkmate)
       break;
@@ -452,13 +439,13 @@ void GoMateThread::operator()()
     double elapsed = start.elapsedSeconds();
     double memory = OslConfig::memoryUseRatio();
     uint64_t node_count = dfpn.nodeCount();
-    total += limit;
     boost::mutex::scoped_lock lk(OslConfig::lock_io);
-    std::cout << "info time " << static_cast<int>(elapsed*1000)
-	      << " nodes " << node_count << " nps " << static_cast<int>(node_count/elapsed)
-	      << " hashfull " << static_cast<int>(memory*1000) << "\n";
+    os << "info time " << static_cast<int>(elapsed*1000)
+       << " nodes " << total+node_count << " nps " << static_cast<int>(node_count/elapsed)
+       << " hashfull " << static_cast<int>(memory*1000) << "\n";
     if (result.isFinal() || elapsed >= seconds || memory > 0.9 || stop_checkmate)
       break;
+    total += limit;
     // estimate: total * std::min(seconds/elapsed, 1.0/memory)
     // next: (estimate - total) / 2 + total
     scale = (total * std::min(seconds/elapsed, 1.0/memory) - total) / 2.0 / step;
@@ -469,28 +456,28 @@ void GoMateThread::operator()()
     std::cerr << "elapsed " << start.elapsedSeconds() << "\n";
   if (! result.isFinal()) {
     boost::mutex::scoped_lock lk(OslConfig::lock_io);
-    std::cout << "checkmate timeout\n" << std::flush;
+    os << "checkmate timeout\n" << std::flush;
     return;
   }
   if (! result.isCheckmateSuccess()) {
     boost::mutex::scoped_lock lk(OslConfig::lock_io);
-    std::cout << "checkmate nomate\n" << std::flush;
+    os << "checkmate nomate\n" << std::flush;
 ;
     return;
   }
   boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  std::cout << "checkmate";
+  os << "checkmate";
   for (size_t i=0; i<pv.size(); ++i)
-    std::cout << " " << record::usi::show(pv[i]);
-  std::cout << "\n" << std::flush;
+    os << " " << record::usi::show(pv[i]);
+  os << "\n" << std::flush;
 }
 
-void do_checkmate(double seconds)
+void do_checkmate(double seconds, std::ostream& os)
 {
-  boost::shared_ptr<boost::thread> (new boost::thread(GoMateThread(seconds)));
+  boost::shared_ptr<boost::thread> (new boost::thread(GoMateThread(seconds, os)));
 }
 
-void play_by_book(int depth_limit)
+void play_by_book(int depth_limit, std::ostream& os)
 {
   static osl::record::opening::WeightedBook book(OslConfig::openingBook());
   if (HashKey(input_state->initial_state)
@@ -510,7 +497,7 @@ void play_by_book(int depth_limit)
 	  && (! state.isIllegal(best_move))) {
 	boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	std::string result = "bestmove " + record::usi::show(best_move);
-	std::cout << result << "\n" << std::flush;
+	os << result << "\n" << std::flush;
 	if (is_verbose())
 	  std::cerr << "sent: " << result << "\n";
 	return;
@@ -518,104 +505,23 @@ void play_by_book(int depth_limit)
     }
   }
   boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  std::cout << "bestmove pass\n" << std::flush;
+  os << "bestmove pass\n" << std::flush;
   if (is_verbose())
     std::cerr << "sent: " << "bestmove pass\n";
 }
 
-void play_declare_win()
+void play_declare_win(std::ostream& os)
 {
   const bool can_declare_win
     = EnterKing::canDeclareWin(input_state->currentState());
   const std::string msg = std::string("bestmove ")
     + (can_declare_win ? "win" : "pass") + "\n" ;
   boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  std::cout << msg << std::flush;
+  os << msg << std::flush;
   if (is_verbose())
     std::cerr << "sent: " << msg;
 }
 
-MoveVector generate_good_moves()
-{
-  game_playing::GameState state(input_state->initial_state);
-  BOOST_FOREACH(Move m, input_state->moves) 
-    state.pushMove(m);
-
-  MoveVector normal_or_win_or_draw, loss;
-  state.generateNotLosingMoves(normal_or_win_or_draw, loss);
-  if (is_verbose() && ! loss.empty()) {
-    std::cerr << "  removed losing move ";
-    BOOST_FOREACH(Move m, loss)
-      std::cerr << record::csa::show(m);
-    std::cerr << "\n";
-  }
-  return normal_or_win_or_draw;
-}
-
-void genmove()
-{
-  MoveVector moves = generate_good_moves();
-  boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  std::cout << "genmove";
-  BOOST_FOREACH(Move move, moves)
-    std::cout << " " << record::usi::show(move);
-  std::cout << "\n";
-}
-
-void genmove_probability(int limit)
-{
-  game_playing::GameState gstate(input_state->initial_state);
-  BOOST_FOREACH(Move m, input_state->moves) 
-    gstate.pushMove(m);
-  const NumEffectState& state= gstate.state();
-  const MoveStack& history = gstate.moveHistory();
-  progress::ml::NewProgress progress(state);
-  MoveLogProbVector moves;
-  if (new_move_probability) {
-    const move_probability::StandardFeatureSet& feature_set
-      = move_probability::StandardFeatureSet::instance();
-    Move threatmate = move_probability::StateInfo::findShortThreatmate
-      (state, history.lastMove());
-    move_probability::StateInfo info(state, progress.progress16(),
-				     history, threatmate);
-    feature_set.generateLogProb(info, moves);
-  } else {
-    const rating::StandardFeatureSet& feature_set
-      = rating::StandardFeatureSet::instance();
-    rating::RatingEnv env;
-    env.history = history;
-    env.make(state, state.pin(state.turn()), state.pin(alt(state.turn())),
-	     progress.progress16());
-    feature_set.generateLogProb(state, env, limit, moves);
-  }
-  for (size_t i=1; i<moves.size(); ++i)
-    if (moves[i].logProb() <= moves[i-1].logProb() && moves[i-1].logProb()+1<=limit)
-      moves[i].setLogProb(moves[i-1].logProb()+1);
-  MoveVector good_moves = generate_good_moves();
-  boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  std::cout << "genmove_probability";
-  BOOST_FOREACH(MoveLogProb move, moves)
-    if (good_moves.isMember(move.move()))
-      std::cout << " " << record::usi::show(move.move()) << " " << move.logProb();
-  std::cout << "\n";
-}
-
-void csashow()
-{
-  NumEffectState state(input_state->currentState());
-  boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  std::cout << state;
-  std::cout << "csashowok\n";
-}
-
-void csamove(std::string str)
-{
-  const NumEffectState state(input_state->currentState());
-  const Move move = record::usi::strToMove(str, state);
-  boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  std::cout << "csamove " << record::csa::show(move) << "\n";
-}
-
 std::string make_filename()
 {
   return OslConfig::gpsusiConf();
@@ -823,11 +729,44 @@ void setOption(std::string& line)
   write_config_file();
 }
 
+void do_stop() 
+{
+  {
+    boost::mutex::scoped_lock lk(go_stop_mutex);
+    if (go_count <= bestmove_count) {
+      std::cerr << "warning stop ignored (go " << go_count
+		<< " times, bestmove " << bestmove_count << " times)\n";
+      return;
+    }
+  }
+  if (last_go_is_checkmate)
+    stop_checkmate = true;
+  else {
+    player->stopSearchNow();
+    player->needRestartNext();
+  }
+  boost::mutex::scoped_lock lk(go_stop_mutex);
+  while (go_count > bestmove_count)
+    go_stop_condition.wait(lk);
+}
+
 #ifdef _WIN32
 void play(std::istream& is, std::ostream& os) __attribute__((noinline, force_align_arg_pointer));
 #endif
 void play(std::istream& is, std::ostream& os)
 {
+  if (udp_log != "" && ! udp_socket) {
+    std::vector<std::string> args;
+    boost::algorithm::split(args, udp_log, boost::algorithm::is_any_of(":"));
+
+    using boost::asio::ip::udp;
+    udp::resolver resolver(io_service);
+    udp::resolver::query query(udp::v4(), args[0], args[1]);
+
+    udp_socket.reset(new udp::socket(io_service));
+    udp_socket->open(udp::v4());
+    udp_endpoint.reset(new udp::endpoint(*resolver.resolve(query)));
+  }
   std::string line;
   std::getline(is, line);
 #ifdef GPSUSIONE
@@ -861,20 +800,39 @@ void play(std::istream& is, std::ostream& os)
     input_log_stream.reset(new std::ofstream(input_log_file.c_str()));
     *input_log_stream << line << "\n" << std::flush;
   }
-    
+  input_state.reset(new UsiState);  
+  UsiResponse lightweight_commands(*input_state,
+				   new_move_probability,
+				   is_verbose());
+  std::string response;
+  response.reserve(2000);
+  bool show_date = false;
   while (true) {
-    if (debug)
+    const bool prompt = (debug || &is != &std::cin);
+    if (prompt && show_date)
       std::cerr << "readyinput\n";
     if (! std::getline(is, line))
       break;
-    if (debug) {
-      const time_t now = time(0);
-      char ctime_buf[64];
-      std::cerr << osl::ctime_r(&now, ctime_buf);
+    if (prompt) {
+      show_date = line.find("echo")>0 && line.find("setoption")>0;
+      if (show_date) {
+	const time_t now = time(0);
+	char ctime_buf[64];
+	std::cerr << osl::ctime_r(&now, ctime_buf);
+      }
       std::cerr << "recv: " << line << "\n" << std::flush;
     }
     if (input_log_stream)
       *input_log_stream << line << "\n" << std::flush;
+    if (udp_socket) {
+      boost::system::error_code ignored_error;
+      line += "\n";
+      udp_socket->send_to(boost::asio::buffer(line.c_str(), line.length()),
+                          *udp_endpoint, 0,
+			  ignored_error);
+      line.resize(line.size()-1);
+    }  
+    
     if (line == "isready") {
       static bool initalized = false;
       if (! initalized) {
@@ -883,6 +841,7 @@ void play(std::istream& is, std::ostream& os)
 	eval::ml::OpenMidEndingEval::setUp();
 	progress::ml::NewProgress::setUp();
 	rating::StandardFeatureSet::instance();
+	lightweight_commands.hasImmediateResponse("genmove_probability", response); // setup
 	initalized = true;
       }
       os << "readyok\n";
@@ -897,23 +856,7 @@ void play(std::istream& is, std::ostream& os)
     if (line.find("usinewgame") == 0)
       continue;
     if (line.find("stop") == 0) {
-      {
-	boost::mutex::scoped_lock lk(go_stop_mutex);
-	if (go_count <= bestmove_count) {
-	  std::cerr << "warning stop ignored (go " << go_count
-		    << " times, bestmove " << bestmove_count << " times)\n";
-	  continue;
-	}
-      }
-      if (last_go_is_checkmate)
-	stop_checkmate = true;
-      else {
-	player->stopSearchNow();
-	player->needRestartNext();
-      }
-      boost::mutex::scoped_lock lk(go_stop_mutex);
-      while (go_count > bestmove_count)
-	go_stop_condition.wait(lk);
+      do_stop();
       continue;
     }
     if (line.find("quit") == 0) {
@@ -929,43 +872,21 @@ void play(std::istream& is, std::ostream& os)
       boost::this_thread::sleep(boost::posix_time::seconds(60));
       continue;
     }
-    if (line.find("genmove_probability") == 0) {
-      int limit = 2000, value;
-      std::istringstream is(line.substr(std::string("genmove_probability").size()));
-      if (is >> value)
-	limit = value;
-      genmove_probability(limit);
-      continue;
-    }
-    if (line.find("genmove") == 0) {
-      genmove();
-      continue;
-    }
-    if (line.find("csashow") == 0) {
-      csashow();
-      continue;
-    }
-    if (line.find("csamove ") == 0) {
-      csamove(line.substr(8));
-      continue;
-    }
-    if (line.find("echo ") == 0) {
+
+    if (lightweight_commands.hasImmediateResponse(line, response)) {
       boost::mutex::scoped_lock lk(OslConfig::lock_io);
-      std::cout << line.substr(5) << "\n";
-      std::cerr << line.substr(5) << "\n";
+      os << response << "\n" << std::flush;
       continue;
     }
 
-    // searc related commands 
+    // search related commands 
     // do not read next search command while previous search is in progress
     while (read_from_file && suspend_reading)
       boost::this_thread::sleep(boost::posix_time::milliseconds(200));
 
     if (line.find("position") == 0) {
-      if (! input_state)
-	input_state.reset(new UsiState);
       try {
-	record::usi::parse(line.substr(8), input_state->initial_state, input_state->moves);
+	input_state->parseUsi(line);
 	if (is_verbose())
 	  std::cerr << "new position " << input_state->moves.size()
 		    << " " << line << "\n";
@@ -989,7 +910,7 @@ void play(std::istream& is, std::ostream& os)
     if (line.find("go mate") == 0) {
       double seconds = 60, millisec;
       std::string option;
-      std::istringstream is(line.substr(std::string("go mate").size()));
+      std::istringstream is(line.substr(strlen("go mate")));
       if (is >> millisec)
 	seconds = std::max(std::min(seconds, millisec/1000.0), 0.5);
       {
@@ -998,63 +919,19 @@ void play(std::istream& is, std::ostream& os)
 	last_go_is_checkmate = true;
 	OslConfig::setDfpnMaxDepth(1600);
       }
-      do_checkmate(seconds);
+      do_checkmate(seconds, os);
       continue;
     }
     if (line.find("go book") == 0) {
       int limit = book_depth, value;
-      std::istringstream is(line.substr(std::string("go book").size()));
+      std::istringstream is(line.substr(strlen("go book")));
       if (is >> value)
 	limit = value;
-      play_by_book(limit);
+      play_by_book(limit, os);
       continue;
     }
     if (line.find("go declare_win") == 0) {
-      play_declare_win();
-      continue;
-    }
-    if (line.find("go window ") == 0) {
-      if (ponder_enabled) {
-	std::string err
-	  = "go window not supported if ponder enabled";
-	std::cerr << err << "\n";
-	  throw std::runtime_error(err);
-      }
-      bwtime.fill(1000);
-      byoyomi = 1000;
-      std::string option;
-      std::istringstream is(line.substr(strlen("go window ")));
-      int alpha, beta;
-      if (! (is >> alpha >> beta)) {
-	std::string err
-	  = "go window: window error ";
-	std::cerr << err << "\n";
-	  throw std::runtime_error(err);
-      }
-      std::cerr <<"window is " << alpha << ' ' << beta << " "
-		<< (bool)is << ' ' << input_turn() << "\n";
-      if (input_turn() == WHITE) {
-	std::swap(alpha, beta);
-	alpha = -alpha;
-	beta = -beta;
-	std::cerr <<"window adjustment " << alpha << ' ' << beta << " " << (bool)is << "\n";
-      }
-      while (is >> option) {
-	if (option == "byoyomi")
-	  is >> byoyomi;
-	else {
-	  std::string err
-	    = "unknown option in go window " + option;
-	  std::cerr << err << "\n";
-	  throw std::runtime_error(err);
-	}
-      }
-      byoyomi_extension = byoyomi;
-      OslConfig::setRootWindow(alpha, beta);
-      suspend_reading = true;
-      do_search();
-      boost::mutex::scoped_lock lk(go_stop_mutex);
-      ++go_count;
+      play_declare_win(os);
       continue;
     }
     if (line.find("go ") == 0) {
@@ -1085,7 +962,7 @@ void play(std::istream& is, std::ostream& os)
 	byoyomi_extension = 0;
       }
       suspend_reading = true;
-      do_search();
+      do_search(os);
       boost::mutex::scoped_lock lk(go_stop_mutex);
       ++go_count;
       continue;
@@ -1096,29 +973,17 @@ void play(std::istream& is, std::ostream& os)
     // non-standard commands
     if (line.find("ignore_moves") == 0) {
       assert(ignore_moves.empty());
-      assert(input_state);
-      if (! input_state) {
-	std::cerr << "error ignore_moves without position\n";
-	continue;
-      }
       if (book_depth > 0) {
 	std::cerr << "error ignore_moves not supported with book\n";
 	continue;
       }
-      std::istringstream is(line);
-      std::string word;
-      is >> word;
-      NumEffectState state(input_state->currentState());
-      while (is >> word) {
- 	ignore_moves.push_back(record::usi::strToMove(word, state));
-      }
+      input_state->parseIgnoreMoves(line, ignore_moves);
       std::cerr << "accept ignore_moves\n";
       if (player)
 	player->needRestartNext();
       continue;
     }
     if (line.find("query progress") == 0) {
-      assert(input_state);
       if (input_state) {
 	NumEffectState state(input_state->currentState());
 	const osl::progress::Effect5x3 progress(state);
@@ -1135,7 +1000,6 @@ void play(std::istream& is, std::ostream& os)
       continue;
     }
     if (line.find("query eval") == 0) {
-      assert(input_state);
       if (input_state) {
 	NumEffectState state(input_state->currentState());
 	const osl::eval::ml::OpenMidEndingEval eval(state);
@@ -1156,7 +1020,6 @@ void play(std::istream& is, std::ostream& os)
       continue;
     }
     if (line.find("query searchtime ") == 0) {
-      assert(input_state);
       if (input_state) {
 	std::istringstream is(line);
 	std::string dummy;
@@ -1187,32 +1050,23 @@ void play(std::istream& is, std::ostream& os)
       continue;      
     }    
     if (line.find("open ") == 0) {
-      if (! input_state)
-	input_state.reset(new UsiState);
-      std::istringstream is(line);
-      std::string dummy, filename;
       bool ok = true;
-      ok &= static_cast<bool>(is >> dummy >> filename);
-      if (ok) {
-	try {
-	  CsaFile csa(filename.c_str());
-	  input_state->initial_state = csa.getInitialState();
-	  input_state->moves = csa.getRecord().getMoves();
-	}
-	catch (record::csa::CsaIOError& e) {
-	  std::cerr << e.what() << "\n";
-	  ok = false;
-	}
+      try {
+	input_state->openFile(line.substr(5));
+      }
+      catch (record::csa::CsaIOError& e) {
+	std::cerr << e.what() << "\n";
+	ok = false;
       }
       if (ok) {
 	boost::mutex::scoped_lock lk(OslConfig::lock_io);
-	std::cout << "position "
-		  << record::usi::show(input_state->initial_state)
-		  << " moves";
+	os << "position "
+	   << record::usi::show(input_state->initial_state)
+	   << " moves";
 	BOOST_FOREACH(Move move, input_state->moves)
-	  std::cout << " " << record::usi::show(move);
-	std::cout << "\n";
-	std::cout << "openok\n";	
+	  os << " " << record::usi::show(move);
+	os << "\n";
+	os << "openok\n";	
       }
       if (! ok)
 	std::cerr << "open failed " << line << "\n";
@@ -1222,21 +1076,25 @@ void play(std::istream& is, std::ostream& os)
   }
   if (read_from_file) {
     std::cerr << "end of input\n";
-    boost::this_thread::sleep(boost::posix_time::seconds(60));
-    if (search_thread) {
-      queue.push(Move());
-      search_thread->join();
-    }
   } else {
-    std::cerr << "stdin closed\n";
+    std::cerr << "input stream closed\n";
   }
+  const int sleep_seconds = 10;
+  std::cerr << "--- trying safe shutdown in "
+	    << sleep_seconds << "s. ---\n";
+  do_stop();
+  if (search_thread) {
+    queue.push(Move());
+    search_thread->join();
+  }
+  boost::this_thread::sleep(boost::posix_time::seconds(sleep_seconds));
 }
 
 int main(int argc, char **argv)
 {
   int num_benchmark, benchmark_seconds;
   po::options_description options("Options");
-  std::string input_file;
+  std::string io_stream;
 #ifndef MINIMAL
   unsigned int eval_random;
 #endif
@@ -1244,6 +1102,7 @@ int main(int argc, char **argv)
   int num_cpus;
 #endif
   double memory_use_percent;
+  bool extended_usi;
   options.add_options()
     ("benchmark", "test search performance")
     ("benchmark-single", "test search performance")
@@ -1254,7 +1113,8 @@ int main(int argc, char **argv)
      po::value<int>(&benchmark_seconds)->default_value(30),
      "seconds for benchmark")
     ("profile", "profile mode (run in 1 thread even when OSL_SMP is defined)")
-    ("read-from-file", po::value<std::string>(&input_file)->default_value(""), "input for debug")
+    ("io-stream", po::value<std::string>(&io_stream)->default_value(""), "input for debug")
+    ("udp-logging", po::value<std::string>(&udp_log)->default_value(""), "host:port for udp logging")
     ("book-width", 
      po::value<int>(&book_width)->default_value(book_width),
      "relative width of book")
@@ -1280,6 +1140,9 @@ int main(int argc, char **argv)
     ("memory-use-percent",
      po::value<double>(&memory_use_percent)->default_value(100.0),
      "percentage for memory use (normally 100)")
+    ("extended-usi",
+     po::value<bool>(&extended_usi)->default_value(false),
+     "use extended usi (currently, pv will be shown in extended format)")
     ("health-check", "test whether data files are properly placed")
     ("help,h", "produce help message")
     ("version", "show version info")
@@ -1354,7 +1217,7 @@ int main(int argc, char **argv)
     return 0;
   }
 
-  OslConfig::setUsiMode();
+  OslConfig::setUsiMode(extended_usi ? OslConfig::ExtendedUSI : OslConfig::PortableUSI);
   OslConfig::setSearchExactValueInOneReply(true); // todo: option
   read_config_file();
 #ifdef OSL_SMP
@@ -1365,10 +1228,32 @@ int main(int argc, char **argv)
 #endif
   setvbuf(stdout, NULL, _IONBF, 0);
   setvbuf(stdin, NULL, _IONBF, 0);
-  if (input_file != "") 
+  if (io_stream.find("tcp:") == 0
+      && std::count(io_stream.begin(), io_stream.end(), ':') == 2)
+  {
+    std::vector<std::string> args;
+    boost::algorithm::split(args, io_stream, boost::algorithm::is_any_of(":"));
+    assert(args.size() == 3);
+    while (1) {
+      std::cerr << "connecting " << std::flush;
+      using boost::asio::ip::tcp;
+      boost::scoped_ptr<tcp::iostream> s
+	(new tcp::iostream(args[1], args[2]));
+      while (! *s) {
+	std::cerr << "." << std::flush;
+	boost::this_thread::sleep(boost::posix_time::seconds(15));
+	s.reset(new tcp::iostream(args[1], args[2]));
+      }
+      std::cerr << "\n" << std::flush;
+      play(*s, *s);
+      // fall back to reconnect in case of server down
+      boost::this_thread::sleep(boost::posix_time::seconds(15));
+    }
+  }
+  else if (io_stream != "") 
   {
     read_from_file = true;
-    std::ifstream is(input_file.c_str());
+    std::ifstream is(io_stream.c_str());
     play(is, std::cout);
   }
   else 
@@ -1389,7 +1274,7 @@ bool ThreadRun::goodAlignment()
   return true;
 }
 
-game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState& initial_state)
+game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState& initial_state, std::ostream& os)
 {
   static osl::record::opening::WeightedBook book(OslConfig::openingBook());
   game_playing::SearchPlayer *search_player = new game_playing::AlphaBeta2OpenMidEndingEvalPlayer;
@@ -1411,6 +1296,12 @@ game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState& initial
   if (node_count_hard_limit)
     search_player->setNodeCountHardLimit(node_count_hard_limit);
 #endif
+  const bool extended_usi = OslConfig::usiMode() == OslConfig::ExtendedUSI;
+  boost::shared_ptr<UsiMonitor> monitor(new UsiMonitor(extended_usi, os));
+  if (udp_socket)
+    monitor->setUdpLogging(udp_socket.get(), udp_endpoint.get());
+
+  search_player->addMonitor(monitor);
   bool use_book = book_depth > 0;
   {
     const SimpleState usual(HIRATE);
