@@ -172,6 +172,8 @@
 #include "../../osl/lib/checkmate/checkmateIfCapture.cc"
 #include "../../osl/lib/checkmate/immediateCheckmate.cc"
 
+#include "../../osl/lib/game_playing/usiState.cc"
+#include "../../osl/lib/game_playing/usiResponse.cc"
 #include "../../osl/lib/game_playing/gameState.cc"
 #include "../../osl/lib/game_playing/computerPlayer.cc"
 #include "../../osl/lib/game_playing/csaLogger.cc"
