@@ -2,12 +2,15 @@
 #define GPSSHOGI_GUI_ABSTRACT_BOARD_H
 #include <QWidget>
 #include <QPixmap>
+#include <QMouseEvent>
 
 #include "osl/piece.h"
 #include "osl/ptype.h"
 #include "osl/player.h"
 #include "osl/square.h"
 
+class QResizeEvent;
+
 namespace gpsshogi
 {
   namespace gui
@@ -18,8 +21,10 @@ namespace gpsshogi
     public:
       AbstractBoard(QWidget *parent = 0);
       QSize sizeHint() const;
+      QSize minimumSize() const;
     protected:
       virtual void paintEvent(QPaintEvent *);
+      virtual void resizeEvent(QResizeEvent *);
     signals:
       void statusChanged();
     protected:
@@ -32,11 +37,15 @@ namespace gpsshogi
       virtual bool reversed() const {
 	return false;
       }
+      virtual void mouseMoveEvent(QMouseEvent *);
       const osl::Square getSquare(int x, int y) const;
       const osl::Piece getPiece(int x, int y);
       const QPoint positionToPoint(const osl::Square position) const;
       const QPixmap pieceToImage(osl::Piece) const;
+      const QString pieceString(osl::Piece) const;
       osl::Piece selectedPiece;
+      QPoint mousePoint;
+      qreal scale_mult;
       static const int STAND_SIZE = 80;
       static const int BOX_SIZE = 40;
       static const int BLACK_STAND_X = STAND_SIZE + BOX_SIZE * 9 + 20;
