@@ -2,6 +2,7 @@
 #define GPSSHOGI_GUI_BOARD_H
 #include "gpsshogi/gui/abstractBoard.h"
 #include "osl/state/numEffectState.h"
+#include "osl/container/moveVector.h"
 
 namespace gpsshogi
 {
@@ -35,10 +36,16 @@ namespace gpsshogi
 	highlight_enabled = on; 
 	update();
       }
+      void highlightBookMove(bool on) { 
+	highlight_book = on; 
+	updateBook();
+	update();
+      }
     protected:
       void paintEvent(QPaintEvent *);
       void mousePressEvent(QMouseEvent *);
       void mouseReleaseEvent(QMouseEvent *);
+      void updateBook();
     public slots:
       void move(osl::Move m);
       void setState(const osl::state::NumEffectState& state, osl::Move last_move=osl::Move());
@@ -60,7 +67,8 @@ namespace gpsshogi
       bool allowManualMovement;
       osl::Square highlight;
       EffectType effect;
-      static bool highlight_enabled;
+      osl::MoveVector book_moves;
+      static bool highlight_enabled, highlight_book;
     };
   }
 }
