@@ -18,15 +18,15 @@ namespace gpsshogi
     ~RecordDB();
 
     static const std::string makeKey(const osl::SimpleState& state);
-    bool get(const std::string& key, PositionInfo& info);
-    bool get(const osl::SimpleState& state, PositionInfo& info);
-    bool put(const std::string& key, const PositionInfo& info);
-    bool put(const osl::SimpleState& state, const PositionInfo& info);
+    bool get(const std::string& key, SquareInfo& info);
+    bool get(const osl::SimpleState& state, SquareInfo& info);
+    bool put(const std::string& key, const SquareInfo& info);
+    bool put(const osl::SimpleState& state, const SquareInfo& info);
     bool optimize();
     size_t size() { return db->size(); }
 
     void initIterator();
-    bool next(osl::record::MiniBoardChar50 &state, PositionInfo &info);
+    bool next(osl::record::MiniBoardChar50 &state, SquareInfo &info);
   };
 }
 
