@@ -4,14 +4,17 @@
 #include <qcursor.h>
 
 #include "gpsshogi/gui/board.h"
-#include "osl/pieceStand.h"
+#include "osl/record/opening/bookInMemory.h"
 #include "osl/state/numEffectState.h"
+#include "osl/pieceStand.h"
+#include <boost/foreach.hpp>
 
 #if QT_VERSION >= 0x040000
 #include <QMouseEvent>
 #endif
 
 bool gpsshogi::gui::Board::highlight_enabled = false;
+bool gpsshogi::gui::Board::highlight_book = false;
 
 gpsshogi::gui::Board::Board(const osl::state::NumEffectState& state,
 			    QWidget *parent)
@@ -24,15 +27,16 @@ gpsshogi::gui::Board::Board(const osl::state::NumEffectState& state,
   palette.setColor(QPalette::Window, color); 
   setPalette(palette); 
   setAutoFillBackground(true);
-  setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 }
 
 void gpsshogi::gui::
 Board::mousePressEvent(QMouseEvent *e)
 {
+  const int x = e->x() / scale_mult;
+  const int y = e->y() / scale_mult;
   if (!allowManualMovement)
     return;
-  osl::Square pos = getSquare(e->x(), e->y());
+  osl::Square pos = getSquare(x, y);
   if (!pos.isValid())
     return;
   if (! pos.isPieceStand())
@@ -46,15 +50,15 @@ Board::mousePressEvent(QMouseEvent *e)
     int pieceNum = -1;
     if (((state.turn() == osl::BLACK && senteView)
 	 || (state.turn() == osl::WHITE && !senteView))
-	&& BLACK_STAND_X <= e->x() && e->x() <= BLACK_STAND_X + BOX_SIZE)
+	&& BLACK_STAND_X <= x && x <= BLACK_STAND_X + BOX_SIZE)
     {
-      pieceNum = (e->y() - BOX_SIZE * 2) / BOX_SIZE;
+      pieceNum = (y - BOX_SIZE) / BOX_SIZE;
     }
     if (((state.turn() == osl::WHITE && senteView)
 	 || (state.turn() == osl::BLACK && !senteView))
-	&& 0 <= e->x() && e->x() <= 0 + BOX_SIZE)
+	&& 0 <= x && x <= 0 + BOX_SIZE)
     {
-      pieceNum = 9 - (e->y() / BOX_SIZE);
+      pieceNum = 9 - (y / BOX_SIZE);
     }
     if (0 <= pieceNum && pieceNum < (int)osl::PieceStand::order.size())
     {
@@ -73,8 +77,7 @@ Board::mousePressEvent(QMouseEvent *e)
   }
   if (!selectedPiece.isEmpty())
   {
-    QCursor cursor(pieceToImage(selectedPiece));
-    setCursor(cursor);
+    mousePoint = e->pos();
     update();
   }
 }
@@ -82,9 +85,11 @@ Board::mousePressEvent(QMouseEvent *e)
 void gpsshogi::gui::
 Board::mouseReleaseEvent(QMouseEvent *e)
 {
+  const int x = e->x() / scale_mult;
+  const int y = e->y() / scale_mult;
   if (!selectedPiece.isEmpty())
   {
-    osl::Square pos = getSquare(e->x(), e->y());
+    osl::Square pos = getSquare(x, y);
     if (pos.isValid() && pos.isOnBoard() && pieceAt(pos).ptype() != osl::KING)
     {
       osl::Move mv = osl::Move::INVALID();
@@ -134,12 +139,21 @@ Board::mouseReleaseEvent(QMouseEvent *e)
   update();
 }
 
+void gpsshogi::gui::
+Board::updateBook()
+{
+  static const osl::BookInMemory& book = osl::BookInMemory::instance();
+  book_moves.clear();
+  book.find(osl::HashKey(state), book_moves);
+}
 
 void gpsshogi::gui::
 Board::move(osl::Move m)
 {
   state.makeMove(m);  
   highlight = m.to();
+  updateBook();
+  
   emit moved(m);
   emit statusChanged();
   update();
@@ -152,6 +166,8 @@ Board::setState(const osl::state::NumEffectState& sstate, osl::Move last_move)
   {
     state = sstate;
     highlight = last_move.isNormal() ? last_move.to() : osl::Square();
+    updateBook();
+
     update();
     emit statusChanged();
   }
@@ -209,6 +225,7 @@ osl::Piece gpsshogi::gui::Board::getStandPiece(osl::Player p, osl::Ptype ptype)
 void gpsshogi::gui::Board::paintEvent(QPaintEvent *pe)
 {
   QPainter painter(this);
+  painter.scale(scale_mult, scale_mult);
   if (effect != NONE)
   {
     osl::state::NumEffectState nState(state);
@@ -244,6 +261,18 @@ void gpsshogi::gui::Board::paintEvent(QPaintEvent *pe)
     QBrush brush(QColor(255, 230, 200));
     painter.fillRect(point.x(), point.y(), BOX_SIZE, BOX_SIZE, brush);
   }
+  if (highlight_book)
+  {
+    BOOST_FOREACH(osl::Move move, book_moves) {
+      QPoint point = positionToPoint(move.to());
+      QBrush brush(QColor(230, 255, 200));
+      painter.fillRect(point.x(), point.y(), BOX_SIZE, BOX_SIZE, brush);
+    }
+  }
   painter.end();
   AbstractBoard::paintEvent(pe);
 }
+// ;;; Local Variables:
+// ;;; mode:c++
+// ;;; c-basic-offset:2
+// ;;; End:
