@@ -44,7 +44,8 @@ void analyze_gradient(const std::string& output_prefix,
 		      size_t kisen_start, size_t num_records);
 void analyze_loss(const std::string& output_prefix,
 		  size_t kisen_start, size_t num_records,
-		  const std::string& gradient_filename);
+		  const std::string& gradient_filename,
+		  size_t num_records_loss_analyses);
 
 boost::scoped_ptr<gpsshogi::Eval> my_eval;
 std::string eval_type;
@@ -64,7 +65,7 @@ int main(int argc, char **argv)
 {
   nice(20);
   
-  size_t kisen_start, num_records;
+  size_t kisen_start, num_records, num_records_loss_analyses;
   std::string output_prefix;
   std::string initial_value, loss_analyses;
   po::options_description options("all_options");
@@ -127,6 +128,9 @@ int main(int argc, char **argv)
     ("loss-analyses",
      po::value<std::string>(&loss_analyses)->default_value(""),
      "analyze loss in existing pv, given filename of gradient as argument")
+    ("num-records-loss-analyses",
+     po::value<size_t>(&num_records_loss_analyses)->default_value(0),
+     "number of records to be analyzed in --loss-analyses (all if 0)")
     ("help", "produce help message")
     ;
 
@@ -182,7 +186,8 @@ int main(int argc, char **argv)
   if (analyze_pv)
     analyze_gradient(output_prefix, kisen_start, num_records);
   else if (loss_analyses != "")
-    analyze_loss(output_prefix, kisen_start, num_records, loss_analyses);
+    analyze_loss(output_prefix, kisen_start, num_records, loss_analyses,
+		 num_records_loss_analyses);
   else
     learn_main(output_prefix, kisen_start, num_records,
 	       cross_start, max_progress);
@@ -357,8 +362,8 @@ void ProcessPV::processInstance(double turn_coef, MoveData& selected, MoveData&
 /* ------------------------------------------------------------------------- */
 struct LossOfPV
 {
-  int thread_id;
-  LossOfPV(int tid) : thread_id(tid)
+  int thread_id, limit_records;
+  LossOfPV(int tid, size_t limit) : thread_id(tid), limit_records(limit)
   {
   }
   void operator()() const;
@@ -376,7 +381,7 @@ void LossOfPV::operator()() const
   int record, position;
   PVVector pv_best, pv;
 
-  int cur_record=-1, cur_position=-1;
+  int cur_record=-1, cur_position=-1, record_done=0;
   NumEffectState state(kisen_file.getInitialState());
   vector<Move> moves;
 
@@ -387,6 +392,9 @@ void LossOfPV::operator()() const
       moves = kisen_file.getMoves(cur_record);
       cur_position = 0;
       state = NumEffectState(kisen_file.getInitialState());
+      ++record_done;
+      if (limit_records > 0 && record_done >= limit_records)
+	break;
     }
     assert(position > cur_position || cur_position == 0);
     while (position > cur_position) {
@@ -467,7 +475,7 @@ public:
     }
     error = errors[0].getAverage();
   }
-  double computeLoss(const valarray_t& w)
+  double computeLoss(const valarray_t& w, size_t limit_records)
   {
     for (size_t i=0; i<num_cpus; ++i) {
       gradient[i].resize(w.size());
@@ -476,7 +484,7 @@ public:
     }
     boost::ptr_vector<boost::thread> threads;
     for (size_t i=0; i<num_cpus; ++i)
-      threads.push_back(new boost::thread(LossOfPV(i)));
+      threads.push_back(new boost::thread(LossOfPV(i, limit_records/num_cpus)));
     for (size_t i=0; i<num_cpus; ++i)
       threads[i].join();
 
@@ -600,7 +608,7 @@ void analyze_gradient(const std::string& output_prefix,
   std::cerr << "make gradient " << ctime(&now);
   solver.makeGradient(w, gradient, error);
   now = time(0);
-  std::cerr << "error =" << error << " writing gradient" << ctime(&now);
+  std::cerr << "error = " << error << " writing gradient" << ctime(&now);
   {
     std::string log_filename = output_prefix + eval_type + "-rand012-analysis.txt";
     std::ofstream os(log_filename.c_str());
@@ -617,12 +625,13 @@ void analyze_gradient(const std::string& output_prefix,
 
 void analyze_loss(const std::string& output_prefix,
 		  size_t kisen_start, size_t num_records,
-		  const std::string& gradient_filename)
+		  const std::string& gradient_filename,
+		  size_t num_records_loss_analyses)
 {
   KisenAnalyzer::distributeJob(num_cpus, &record_configs[0], kisen_start, num_records,
 			       kisen_filenames, min_rating);
 
-  valarray_t w(my_eval->dimension()), gradient(w.size());
+  valarray_t w(my_eval->dimension()), gradient(w.size()), stable_w(w.size());
   my_eval->saveWeight(&w[0]);
   {
     std::ifstream is(gradient_filename.c_str());
@@ -631,10 +640,15 @@ void analyze_loss(const std::string& output_prefix,
     if (!is)
       throw std::runtime_error("read error "+gradient_filename);
   }
+  {
+    boost::scoped_ptr<Eval> eval(EvalFactory::newEval(eval_type));
+    eval->saveWeight(&stable_w[0]);
+  }
+  
   GradientDescent solver;
   time_t now = time(0);
   std::cerr << "comute loss " << ctime(&now);
-  double loss = solver.computeLoss(w);
+  double loss = solver.computeLoss(w, num_records_loss_analyses);
   now = time(0);
   std::cerr << "error =" << loss << ctime(&now);
   std::cout << "0 0 0 0 " << std::setprecision(10) << loss << "\n" << std::flush;
@@ -642,13 +656,13 @@ void analyze_loss(const std::string& output_prefix,
   valarray_t modified = w;
   for (int i=0; i<2048; ++i) {
     int id = osl::random() % w.size();
-    while (w[id] == 0 || gradient[id] == 0)
+    while ((w[id] == 0 && stable_w[id] == 0) || gradient[id] == 0)
       id = osl::random() % w.size();
     for (int d=1; d<=8; d*=2) {
       modified[id] = w[id] + d*((gradient[id]>0) ? -1 : 1);
       my_eval->setWeightScale(&modified[0], 1.0);
       
-      double loss = solver.computeLoss(modified);
+      double loss = solver.computeLoss(modified, num_records_loss_analyses);
       std::cout << id << ' ' << d << ' ' << gradient[id] << ' ' << w[id]
 		<< ' ' << loss << "\n" << std::flush;
     }
