@@ -834,7 +834,9 @@ void analyze(const std::string& output_prefix,
 	  os << "  " << i << " " << my_eval->flatValue(i);
 	  boost::tuple<std::string, int, int> name_index_dim = my_eval->findFeature(i);
 	  os << "  " << name_index_dim.get<0>() << " " << name_index_dim.get<1>()
-	     << " / " << name_index_dim.get<2>()<< "\n";
+	     << " / " << name_index_dim.get<2>() << "  ";
+	  os << my_eval->describe(name_index_dim.get<0>(), name_index_dim.get<1>())
+	     << "\n";
 	  watch_list.push_back(i);
 	}
       }
