@@ -8,6 +8,7 @@
 #include "osl/eval/progressEval.h"
 #include "osl/misc/fixedCapacityVector.h"
 #include "osl/stl/stack.h"
+#include <boost/lexical_cast.hpp>
 #include <fstream>
 #include <iostream>
 #include <iomanip>
@@ -71,6 +72,18 @@ Eval::maxActive() const
   return dimension();
 }    
 
+const std::string gpsshogi::
+Eval::describe(const std::string& feature, size_t local_index) const
+{
+  return "not implemented";
+}
+
+const std::string gpsshogi::
+Eval::describeAll(const std::string& feature) const
+{
+  return "not implemented";
+}
+
 boost::tuple<std::string, int, int> 
 gpsshogi::Eval::findFeature(size_t /*index*/) const
 {
@@ -277,6 +290,12 @@ EvalComponent::showSummary(std::ostream& os) const
      << " ave(abs) " << 1.0*sum/count << " " << 1.0*sum/dimension() << "\n";
 }
 
+const std::string gpsshogi::
+EvalComponent::describe(size_t local_index) const
+{
+  return "EvalComponent not implemented";
+}
+
 
 
 
@@ -340,6 +359,12 @@ EvalComponentMulti::pieceValue(const NumEffectState&, Piece, const MultiWeights&
   return MultiInt();
 }
 
+const std::string gpsshogi::
+EvalComponentMulti::describe(size_t local_index) const
+{
+  return "EvalComponentMulti";
+}
+
 /* ------------------------------------------------------------------------- */
 
 gpsshogi::
@@ -385,6 +410,12 @@ FeaturesOneNonUniq::pieceValue(const NumEffectState&, Piece, const MultiWeights&
   return MultiInt();
 }
 
+const std::string gpsshogi::
+FeaturesOneNonUniq::describe(size_t local_index) const
+{
+  return "FeaturesOneNonUniq";
+}
+
 /* ------------------------------------------------------------------------- */
 
 gpsshogi::
@@ -678,6 +709,13 @@ EvalComponentStages::makeValue(const index_list_t& features) const
   return  result;
 }
 
+const std::string gpsshogi::
+EvalComponentStages::describe(size_t local_index) const
+{
+  return delegate->describe(local_index % one_dim) + " stage "
+    + boost::lexical_cast<std::string>(local_index/one_dim);
+}
+
 
 /* ------------------------------------------------------------------------- */
 
