@@ -60,6 +60,8 @@ namespace gpsshogi
     virtual void setWeightScale(const double*, double)=0;
     virtual size_t lambdaStart() const=0;
     virtual int flatValue(size_t index) const=0;
+    virtual const std::string describe(const std::string& feature, size_t local_index) const;
+    virtual const std::string describeAll(const std::string& feature) const;
     virtual boost::tuple<std::string, int, int> findFeature(size_t index) const;
     virtual size_t dimension() const=0;
     virtual size_t maxActive() const;
@@ -140,6 +142,7 @@ namespace gpsshogi
     void convert(const index_list_t&, double& value,
 		 vector<std::pair<int, double> >&, int offset) const;
     void showSummary(std::ostream&) const;
+    virtual const std::string describe(size_t local_index) const;
   };
 
   /** MultiIntを使うfeature 重みは外部に持つ */
@@ -163,6 +166,7 @@ namespace gpsshogi
     virtual void showSummary(std::ostream&, const MultiWeights& weights) const;
     virtual void showAll(std::ostream&, const MultiWeights& weights) const;
     virtual MultiInt pieceValue(const NumEffectState&, Piece, const MultiWeights&) const;
+    virtual const std::string describe(size_t local_index) const;
   };
 
   class FeaturesOneNonUniq
@@ -188,6 +192,7 @@ namespace gpsshogi
 			    int n,
 			    std::ostream &os) const;
     virtual MultiInt pieceValue(const NumEffectState&, Piece, const MultiWeights&) const;
+    virtual const std::string describe(size_t local_index) const;
   };
 
   /** MultiIntを使うfeature 重みは自分で持つ FeaturesOneNonUniq を活用 */
@@ -334,6 +339,7 @@ namespace gpsshogi
       }
     }
     MultiInt makeValue(const index_list_t&) const;
+    const std::string describe(size_t local_index) const;
   };
 
   class EvalComponentStagesBW : public EvalComponentStages
