@@ -1,5 +1,8 @@
 #include "eval/minorPiece.h"
 #include "osl/container/pieceVector.h"
+#include "osl/effect_util/additionalEffect.h"
+#include "osl/record/csa.h"
+#include <boost/lexical_cast.hpp>
 
 bool gpsshogi::
 SilverRetreat::canRetreat(const osl::state::NumEffectState &state,
@@ -282,6 +285,23 @@ SilverFork::matchGold(const NumEffectState& state, Piece gold,
   }
   return std::make_pair(0,0);
 }
+const std::string gpsshogi::
+SilverFork::describe(size_t local_index) const
+{
+  std::string ret = (local_index % 2) ? "turn " : "";
+  local_index /= 2;
+  if (local_index == 0)
+    return ret + "Rook Rook";
+  if (local_index == 1)
+    return ret + "Rook Gold+support";
+  if (local_index == 2)
+    return ret + "Rook Gold";
+  if (local_index == 3)
+    return ret + "Gold Gold";
+  assert(local_index == 4);
+  return ret + "Gold Gold+recapure";
+}
+
 void gpsshogi::
 SilverFork::featuresOneNonUniq
 (const NumEffectState &state, IndexCacheI<MaxActiveWithDuplication> &features) const
@@ -449,6 +469,18 @@ BishopRookFork::featuresOneNonUniq
   }
 }
 
+const std::string gpsshogi::
+BishopRookFork::describe(size_t local_index) const
+{
+  std::string ret = (local_index % 2) ? "turn " : "";
+  local_index /= 2;
+  ret += (local_index >= DROP_DIM) ? "rook fork " : "bishop fork ";
+  local_index %= DROP_DIM;
+  const int a = (local_index / PTYPE_SIZE), b = (local_index % PTYPE_SIZE);
+  return ret + Ptype_Table.getCsaName(static_cast<Ptype>(a))
+    + Ptype_Table.getCsaName(static_cast<Ptype>(b));
+}
+
 
 
 bool gpsshogi::
@@ -1529,6 +1561,12 @@ int gpsshogi::PtypeY::index(const Player player,
   const int y = (player == BLACK ? pos.y() : 10 - pos.y()) - 1;
   return ptype * 9 + y;
 }
+const std::string gpsshogi::PtypeY::
+describe(size_t local_index) const
+{
+  return osl::record::csa::show(static_cast<Ptype>(local_index/9))
+    + " y=" + boost::lexical_cast<std::string>((local_index%9)+1);
+}
 
 osl::MultiInt gpsshogi::PtypeY::eval(const NumEffectState &state, const MultiWeights& weights,
 				     CArray<MultiInt,2>& /*saved_state*/) const
@@ -2125,3 +2163,4 @@ SilverAdvance26::featuresOneNonUniq
 // ;;; c-basic-offset:2
 // ;;; End:
 
+
