@@ -1,7 +1,7 @@
 /* minorPiece.h
  */
-#ifndef _GPSSSHOGI_SAMPLE_EVAL_LIB_MINORPIECE_H
-#define _GPSSSHOGI_SAMPLE_EVAL_LIB_MINORPIECE_H
+#ifndef GPSSSHOGI_SAMPLE_EVAL_LIB_MINORPIECE_H
+#define GPSSSHOGI_SAMPLE_EVAL_LIB_MINORPIECE_H
 
 #include "eval/eval.h"
 
@@ -86,6 +86,7 @@ namespace gpsshogi
     void featuresOneNonUniq(const NumEffectState &state,
 			    IndexCacheI<MaxActiveWithDuplication> &) const;
     const std::string name() const { return "SilverFork"; };
+    const std::string describe(size_t local_index) const;
   };
   class BishopRookFork : public FeaturesOneNonUniq
   {
@@ -107,6 +108,7 @@ namespace gpsshogi
     {
       return bishopIndex(a,b) + DROP_DIM;
     }
+    const std::string describe(size_t local_index) const;
   private:
     static bool findDropInLine(const NumEffectState& state, Player defense, 
 			       const Square a, const Square b, Piece king);
@@ -491,6 +493,7 @@ namespace gpsshogi
 		  int offset) const;
     void showSummary(std::ostream &os, const MultiWeights& weights) const;
     const std::string name() const { return "PtypeY"; };
+    const std::string describe(size_t local_index) const;
   };
 
   class PtypeX : public EvalComponentMulti
@@ -703,7 +706,7 @@ namespace gpsshogi
   };
 }
 
-#endif /* _GPSSSHOGI_SAMPLE_EVAL_LIB_MINORPIECE_H */
+#endif /* GPSSSHOGI_SAMPLE_EVAL_LIB_MINORPIECE_H */
 // ;;; Local Variables:
 // ;;; mode:c++
 // ;;; c-basic-offset:2
