@@ -22,6 +22,7 @@
 #include "osl/oslConfig.h"
 
 #include <boost/filesystem.hpp>
+#include <boost/lexical_cast.hpp>
 #include <iomanip>
 #include <fstream>
 #include <iostream>
@@ -646,6 +647,72 @@ OpenMidEndingBase::debug(const NumEffectState& state) const
     std::cerr << triple[i].name() << " " << triple[i].evalMulti(state, tmp)[0] << "\n";
 }
 
+const std::string gpsshogi::
+OpenMidEndingBase::describe(const std::string& feature, size_t local_index) const
+{
+  for (size_t i=0; i<flat.size(); ++i) {
+    if (flat[i].name() == feature) {
+      if (local_index >= flat[i].dimension())
+	return "index error";
+      return flat[i].describe(local_index)
+	+ " value " + boost::lexical_cast<std::string>(flat[i].value(local_index));
+    }
+  }
+  for (size_t i=0; i<legacy.size(); ++i) {
+    if (legacy[i].name() == feature) {
+      const int dim = legacy[i].dimensionOne();
+      if (local_index * EvalStages >= dim)
+	return "index error";
+      const int value = legacy_weights[i].value(local_index%dim)
+	[local_index/dim];	  
+      return legacy[i].describe(local_index)
+	+ " stage " + boost::lexical_cast<std::string>(local_index/dim)
+	+ " value " + boost::lexical_cast<std::string>(value);
+    }
+  }
+  for (size_t i=0; i<triple.size(); ++i) {
+    if (triple[i].name() == feature) {
+      if (local_index >= triple[i].dimension())
+	return "index error";
+      return triple[i].describe(local_index)
+	+ " value " + boost::lexical_cast<std::string>(triple[i].value(local_index));
+    }
+  }
+  return "not found";
+}
+
+const std::string gpsshogi::
+OpenMidEndingBase::describeAll(const std::string& feature) const
+{
+  for (size_t i=0; i<flat.size(); ++i) {
+    if (flat[i].name() == feature) {
+      std::string all;
+      for (size_t j=0; j<flat[i].dimension(); ++j)
+	all += describe(feature, j) + "\n";
+      return all;
+    }
+  }
+  for (size_t i=0; i<legacy.size(); ++i) {
+    if (legacy[i].name() == feature) {
+      const int dim = legacy[i].dimensionOne();
+      std::string all;
+      for (int s=0; s<EvalStages; ++s) 
+	for (int j=0; j<dim; ++j)
+	  all += describe(feature, s*dim + j) + "\n";
+      return all;
+    }
+  }
+  for (size_t i=0; i<triple.size(); ++i) {
+    if (triple[i].name() == feature) {
+      std::string all;
+      for (int j=0; j<triple[i].dimension(); ++j)
+	all += describe(feature, j) + "\n";
+      return all;
+    }
+  }
+  return "not found";
+}
+
 /* ------------------------------------------------------------------------- */
 
 gpsshogi::OpenMidEndingForTest::
@@ -1210,6 +1277,12 @@ OslOpenMidEnding::findFeature(size_t index) const
   return stable_eval->findFeature(index);
 }
 
+const std::string gpsshogi::
+OslOpenMidEnding::describe(const std::string& feature, size_t local_index) const
+{
+  return stable_eval->describe(feature, local_index);
+}
+
 /* ------------------------------------------------------------------------- */
 // ;;; Local Variables:
 // ;;; mode:c++
