@@ -36,6 +36,8 @@ namespace gpsshogi
     void setWeightScale(const double*, double);
     void saveWeight(double*) const;
 
+    const std::string describe(const std::string& feature, size_t local_index) const;
+    const std::string describeAll(const std::string& feature) const;
     int flatValue(size_t index) const {
       for (size_t i=0; i<flat.size(); ++i) {
 	if (index < flat[i].dimension())
@@ -202,6 +204,7 @@ namespace gpsshogi
     int endgameValue(const NumEffectState& state) const;
     int progressIndependentValue(const NumEffectState& state) const;
     boost::tuple<std::string, int, int> findFeature(size_t index) const;
+    const std::string describe(const std::string& feature, size_t local_index) const;
   };
 }
 
