@@ -166,6 +166,8 @@ Board::showEval(const std::string& name) const
     osl::eval::ml::OpenMidEndingEval eval(state);
     std::cout << IconvConvert::eucToLang("ɾ: ")
 	      << eval.value() << " ("
+	      << eval.value()/(eval.captureValue(newPtypeO(osl::WHITE,osl::PAWN))/2)
+	      << ")" << " ("
 	      << eval.openingValue()*progress.maxProgress() << " "
 	      << eval.midgameValue()*progress.maxProgress() << " "
 	      << eval.midgame2Value()*progress.maxProgress() << " "
