@@ -3,13 +3,13 @@
 #include "searchdialogimpl.h"
 #include "quiescenceTree.h"
 #include "moveGeneratorDialog.h"
+#include "searchMonitor.h"
 
 #include "osl/search/simpleHashTable.h"
 #include "osl/search/simpleHashRecord.h"
 #include "osl/search/moveGenerator.h"
 #include "osl/search/fixedEval.h"
 #include "osl/search/searchState2.h"
-#include "osl/search/searchMonitor.h"
 #include "osl/search/analyzer/recordSet_.h"
 #include "osl/eval/ml/openMidEndingEval.h"
 #include "osl/game_playing/alphaBetaPlayer.h"
@@ -641,64 +641,6 @@ bool AnalysisViewer::analyze(const osl::state::SimpleState& s,
   return analyzeWithSavedValue(s, moves);
 }
 
-class ViewerSearchMonitor : public osl::search::SearchMonitor
-{
-  QLabel *label;
-  QTextCodec *codec;
-  const osl::NumEffectState& root;
-  osl::Move root_move;
-  size_t node_count;
-  double elapsed;
-  int pv_value;
-  std::vector<osl::Move> pv;
-public:
-  ViewerSearchMonitor(QLabel *l, const osl::NumEffectState& state)
-    : label(l), codec(QTextCodec::codecForName("EUC-JP")), root(state),
-      node_count(0), elapsed(0)
-  {
-  }
-  void showPV(int /*depth*/, size_t node_count, double elapsed, int value, osl::Move cur, const osl::Move *first, const osl::Move *last)
-  {
-    pv.clear();
-    if (first == last || cur != *first)
-      pv.push_back(cur);
-    pv.insert(pv.end(), first, last);
-    pv_value = value * osl::playerToMul(root.turn());
-    this->node_count = node_count;
-    this->elapsed = elapsed;
-    update();
-  }
-  void rootMove(osl::Move cur)
-  {
-    root_move = cur;
-    update();
-  }
-  void rootFirstMove(osl::Move cur)
-  {
-    root_move = cur;
-    update();
-  }
-  void timeInfo(size_t node_count, double elapsed)
-  {
-    this->node_count = node_count;
-    this->elapsed = elapsed;
-    update();
-  }		      
-  void update()
-  {
-    if (! root_move.isNormal())
-      return;
-    std::string euc_pv = osl::record::ki2::show(&*pv.begin(), &*pv.end(), root);
-    std::string euc_move = osl::record::ki2::show(root_move, root);
-    label->setText(QString::fromUtf8("探索中 %1 節点 %2 %3秒\n%4 %5")
-		   .arg(codec->toUnicode(euc_move.c_str(), euc_move.length()))
-		   .arg(node_count).arg(elapsed, 0, 'g', 3)
-		   .arg(pv_value, 5)
-		   .arg(codec->toUnicode(euc_pv.c_str(), euc_pv.length()))
-      );
-  }
-};
-
 bool AnalysisViewer::analyzeWithSavedValue(const osl::state::SimpleState& s,
 					   const osl::stl::vector<osl::Move> &moves)
 {
@@ -775,13 +717,8 @@ bool AnalysisViewer::analyzeWithSavedValue(const osl::state::SimpleState& s,
       progress.setValue(time);
       break;
     }
-#if QT_VERSION >= 0x040000
     if (thread.isFinished())
       break;
-#else
-    if (thread.finished())
-      break;
-#endif
   }
 
   thread.wait();
@@ -850,7 +787,7 @@ const osl::search::SimpleHashRecord *AnalysisViewer::selectedRecord()
 
 void AnalysisViewer::openMoves(const QModelIndex &index)
 {
-  QVariant v = pvModel->data(index, PvModel::FirstMoveRole);
+  QVariant v = sortedPvModel->data(index, PvModel::FirstMoveRole);
   osl::Move move = osl::Move::makeDirect(v.toInt());
 
   for (SearchMoveTreeItem *child = (SearchMoveTreeItem *)moveTree->firstChild();
