@@ -31,11 +31,11 @@ public:
   virtual osl::Player turn() const {
     return osl::BLACK;
   }
-  virtual const osl::state::SimpleState& getState();
+  virtual const osl::NumEffectState& getState();
   virtual void toggleOrientation() {
   }
-  virtual osl::state::SimpleState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &/*moves*/) {
-    return state;
+  virtual osl::NumEffectState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &/*moves*/) {
+    return getState();
   }
   virtual QWidget *moveGenerateDialog() {
     return NULL;
@@ -45,10 +45,16 @@ private slots:
   void processText();
   void reserveToBlack();
   void reserveToWhite();
+  void blackTurn(bool);
+  void whiteTurn(bool);
 private:
-  osl::state::SimpleState state;
+  osl::NumEffectState state;
   osl::Player player;
   gpsshogi::gui::EditBoard2 *board;
   QLineEdit *line_edit;
 };
 #endif // _BOARDEDITOR2_H
+// ;;; Local Variables:
+// ;;; mode:c++
+// ;;; c-basic-offset:2
+// ;;; End:
