@@ -138,6 +138,10 @@ void BoardTabChild::highlightLastMove(bool on)
 {
   board->highlightLastMove(on);
 }
+void BoardTabChild::highlightBookMove(bool on)
+{
+  board->highlightBookMove(on);
+}
 
 QWidget *BoardTabChild::moveGenerateDialog()
 {
