@@ -1,11 +1,13 @@
 #include "kifuViewer.h"
 #include "moveList.h"
 
-#include "gpsshogi/gui/board.h"
 #include "moveGeneratorDialog.h"
 #include "kifuAnalyzer.h"
 #include "dualEvaluationDialog.h"
 
+#include "gpsshogi/gui/board.h"
+#include "gpsshogi/gui/util.h"
+
 #include "osl/search/searchMonitor.h"
 #include "osl/record/csaRecord.h"
 #include "osl/record/csaString.h"
@@ -13,20 +15,12 @@
 #include "osl/record/kisen.h"
 #include "osl/record/ki2.h"
 
-#include "gpsshogi/gui/util.h"
-
 #include <qlayout.h>
 #include <QListWidget>
 #include <QTableWidget>
 #include <QThread>
-#if QT_VERSION >= 0x040000
 #include <Q3ListView>
 #include <QDateTime>
-#else
-#include <qlistbox.h>
-#include <qlistview.h>
-#define Q3ListViewItem QListViewItem
-#endif
 #include <qtextcodec.h>
 #include <qtimer.h>
 #include <qfile.h>
@@ -37,6 +31,8 @@
 #include <qlabel.h>
 #include <qpushbutton.h>
 
+#include <iostream>
+
 volatile bool KifuViewer::analyzeOnlineEnabled = false;
 volatile bool KifuViewer::stateForAnalysisChanged = false;
 boost::scoped_ptr<osl::NumEffectState> KifuViewer::stateForAnalysis;
@@ -390,11 +386,16 @@ public:
   }
   void showPV(const osl::HashKey& key,
 	      int depth, size_t node_count, double elapsed, int value,
-	      const std::vector<osl::Move>& pv, const char *additional_message="")
+	      const std::vector<osl::Move>& pv, const char *additional_message="",
+	      const std::vector<char> *pv_threatmate=0)
   {
     if (closed || key != root_key)
       return;
-    std::string euc_pv = osl::record::ki2::show(&*pv.begin(), &*pv.end(), root);
+    std::string euc_pv = pv_threatmate
+      ? osl::record::ki2::show(&*pv.begin(), &*pv.end(),
+			       &*pv_threatmate->begin(), &*pv_threatmate->end(),
+			       root)
+      : osl::record::ki2::show(&*pv.begin(), &*pv.end(), root);
     {
       QMutexLocker lk(&analysisOnlineMutex);
       pvList->setUpdatesEnabled(false);
@@ -465,7 +466,6 @@ public:
 };
 
 QRect KifuViewer::AnalysisOnlineDialog::lastGeometry(100, 100, 700, 300);
-#include <iostream>
 class OnlineSearchMonitor : public osl::search::SearchMonitor
 {
   KifuViewer::AnalysisOnlineDialog *dialog;
@@ -480,16 +480,23 @@ public:
       cur_depth(0)
   {
   }
-  void showPV(int depth, size_t node_count, double elapsed, int value, osl::Move cur, const osl::Move *first, const osl::Move *last)
+  void showPV(int depth, size_t node_count, double elapsed, int value, osl::Move cur, const osl::Move *first, const osl::Move *last,
+	      const bool *threatmate_first, const bool *threatmate_last)
   {
     std::vector<osl::Move> pv;
-    if (first == 0 || *first != cur)
+    std::vector<char> pv_threatmate;
+    if (first == 0 || *first != cur) {
       pv.push_back(cur);
-    while (first != last)
+      pv_threatmate.push_back(false);
+    }
+    while (first != last) {
       pv.push_back(*first++);
+      if (threatmate_first != threatmate_last)
+	pv_threatmate.push_back(*threatmate_first++);
+    }
     cur_depth = std::max(cur_depth, depth);
     if (cur_depth >= MinimumDepth || abs(value) >= 1000000)
-      dialog->showPV(key, depth, node_count, elapsed, value, pv);
+      dialog->showPV(key, depth, node_count, elapsed, value, pv, "", &pv_threatmate);
   }
   void rootMove(osl::Move cur)
   {
