@@ -28,7 +28,7 @@ private:
   CopyRateLabel *progressLabel;
   CopyRateLabel *threatmateLabel;
   QAction *flipAction;
-  QAction *effectAction, *lastMoveAction, *analyzeOnlineAction;
+  QAction *effectAction, *lastMoveAction, *bookMoveAction, *analyzeOnlineAction;
   CopyLabel *filenameLabel;
   CopyRateLabel *proDBLabel, *blackGpsDBLabel, *whiteGpsDBLabel;
 };
