@@ -2,7 +2,7 @@
 #define _TABCHILD_H
 #include <qwidget.h>
 #include "osl/player.h"
-#include "osl/state/simpleState.h"
+#include "osl/state/numEffectState.h"
 #include "osl/stl/vector.h"
 
 class TabChild : public QWidget
@@ -17,9 +17,9 @@ public:
   virtual void toLastState() = 0;
   virtual int moveCount() const = 0;
   virtual osl::Player turn() const = 0;
-  virtual const osl::state::SimpleState& getState() = 0;
+  virtual const osl::NumEffectState& getState() = 0;
   virtual void toggleOrientation() = 0;
-  virtual osl::state::SimpleState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &moves) = 0;
+  virtual osl::NumEffectState getStateAndMovesToCurrent(osl::stl::vector<osl::Move> &moves) = 0;
   virtual QString getFilename() {
     return QString();
   }
@@ -38,6 +38,8 @@ public:
   }
   virtual void highlightLastMove(bool) {
   }
+  virtual void highlightBookMove(bool) {
+  }
   virtual QWidget *moveGenerateDialog() = 0;
 };
 
