@@ -2,34 +2,28 @@
 #include "tabchild.h"
 #include "analysisViewer.h"
 #include "checkmateViewer.h"
-#ifdef ENABLE_NTESUKI
-#include "ntesukiViewer.h"
-#endif
 #include "kifuViewer.h"
 #include "quiescenceViewer.h"
 #include "networkViewer.h"
-#include "piecePairDialog.h"
 #include "moveGeneratorDialog.h"
 #include "ui_kisendialog4.h"
 #include "ui_networkdialog4.h"
 #include "kifuFile.h"
 #include "evalDebug.h"
+#include "kisenModel.h"
+#include "progressDebug.h"
 
-#if QT_VERSION < 0x040000
-#include "boardEditor.h"
-#else
 #include "httpwindow.h"
-#endif
 #include "boardEditor2.h"
 
 #include "osl/search/simpleHashTable.h"
 #include "osl/eval/progressEval.h"
-#include "osl/eval/ml/openMidEndingEval.h"
 #include "osl/state/numEffectState.h"
 #include "osl/progress/effect5x3.h"
 #include "osl/record/kisen.h"
 #include "osl/record/kakinoki.h"
 #include "osl/record/csa.h"
+#include "osl/record/csaString.h"
 #include "osl/search/simpleHashRecord.h"
 #include "osl/threatmate/mlPredictor.h"
 #include "osl/threatmate/treePredictor.h"
@@ -41,11 +35,11 @@
 #include <qtextcodec.h>
 #include <qlistview.h>
 #include <qpushbutton.h>
+#include <QClipboard>
 
 #include <boost/scoped_ptr.hpp>
 
 #include <stdexcept>
-#include <iostream>
 
 Viewer::Viewer(QWidget *parent, const char *name)
   : QTabWidget(parent, name), chatIcon(false)
@@ -58,26 +52,14 @@ Viewer::Viewer(QWidget *parent, const char *name)
   analysisViewer = 0;
   checkmateViewer = new CheckmateViewer(this);
   addTab(checkmateViewer, "Check Search");
-#ifdef ENABLE_NTESUKI
-  ntesukiViewer = new NtesukiViewer(this);
-  addTab(ntesukiViewer, "Ntesuki Search");
-#endif
   quiescenceViewer = new QuiescenceViewer(this);
   addTab(quiescenceViewer, "Quiescence Search");
   networkViewer = new NetworkViewer(this);
   addTab(networkViewer, "Network");
-#if QT_VERSION < 0x040000
-  boardEditor = new BoardEditor(this);
-  addTab(boardEditor, "Edit");
-#endif
   editor = new BoardEditor2(this);
   addTab(editor, "Edit Board");
 
-#if QT_VERSION >= 0x040000
   QPixmap close = QPixmap(":/images/close.png");
-#else
-  QPixmap close = QPixmap::fromMimeSource("close.png");
-#endif
 
   QPushButton *closeButton = new QPushButton(close, "", this);
   setCornerWidget(closeButton);
@@ -90,19 +72,10 @@ Viewer::Viewer(QWidget *parent, const char *name)
 	  this, SIGNAL(analyzeOnlineDisabled()));
   connect(checkmateViewer, SIGNAL(statusChanged()),
 	  this, SIGNAL(statusChanged()));
-#ifdef ENABLE_NTESUKI
-  connect(ntesukiViewer, SIGNAL(statusChanged()),
-	  this, SIGNAL(statusChanged()));
-#endif
   connect(quiescenceViewer, SIGNAL(statusChanged()),
 	  this, SIGNAL(statusChanged()));
   connect(networkViewer, SIGNAL(statusChanged()),
 	  this, SIGNAL(statusChanged()));
-#if QT_VERSION < 0x040000
-  connect(boardEditor, SIGNAL(statusChanged()),
-	  this, SIGNAL(statusChanged()));
-#endif
-
   connect(closeButton, SIGNAL(clicked()),
 	  this, SLOT(closeTab()));
   connect(this, SIGNAL(currentChanged(QWidget *)),
@@ -127,11 +100,20 @@ const viewer::EvalInfo Viewer::evalState()
       nState.kingPiece(osl::WHITE).isOnBoard())
   {
     osl::eval::ml::OpenMidEndingEval eval(nState);
-    result.total =
-      eval.value() / osl::progress::ml::NewProgress::maxProgress();;
+#ifdef EVAL_QUAD
+    const int piece_scale = osl::progress::ml::NewProgress::maxProgress()/3;
+#else
+    const int piece_scale = osl::progress::ml::NewProgress::maxProgress();
+#endif
+    result.total = eval.value() / piece_scale;
     result.opening = eval.openingValue();
-    result.mid = eval.midgameValue();
+    result.mid1 = eval.midgameValue();
+#ifdef EVAL_QUAD
+    result.mid2 = eval.midgame2Value();
+#endif
     result.ending  = eval.endgameValue();
+    osl::eval::PieceEval piece(nState);
+    result.tension = result.total - piece.value();
   }
   return result;
 }
@@ -139,6 +121,7 @@ const viewer::EvalInfo Viewer::evalState()
 void Viewer::progressOfState(osl::stl::vector<int>& out)
 {
   TabChild *child = (TabChild *)currentPage();
+  out.clear();
   // TODO: better abstraction needed
   if (currentPage() == editor)
   {
@@ -154,20 +137,11 @@ void Viewer::progressOfState(osl::stl::vector<int>& out)
     out.push_back(progress.progress16().value());
     out.push_back(progress.progress16(osl::BLACK).value());
     out.push_back(progress.progress16(osl::WHITE).value());
-    out.push_back(progress.progressAttack(osl::BLACK).value()-8);
-    out.push_back(progress.progressDefense(osl::BLACK).value()-8);
-    out.push_back(progress.progressAttack(osl::WHITE).value()-8);
-    out.push_back(progress.progressDefense(osl::WHITE).value()-8);
   }
   else
   {
-    out.push_back(0);
-    out.push_back(0);
-    out.push_back(0);
-    out.push_back(0);
-    out.push_back(0);
-    out.push_back(0);
-    out.push_back(0);
+    for (int i = 0; i < 3; i++)
+      out.push_back(0);
   }
 }
 
@@ -256,37 +230,6 @@ void Viewer::openEvalGraph()
   showPage(kifuViewer);
 }
 
-class KisenItem : Q3ListViewItem
-{
-public:
-  KisenItem(Q3ListView *parent, int index, QString str1, QString str2,
-	    QString str3, QString str4, QString str5)
-    : Q3ListViewItem(parent, str1, str2, str3, str4, str5), index(index) {
-  }
-  int getIndex() const {
-    return index;
-  }
-  int compare(Q3ListViewItem *i, int col, bool ascending) const;
-private:
-  int index;
-};
-
-int KisenItem::compare(Q3ListViewItem *i, int col, bool ascending) const
-{
-  KisenItem *item = (KisenItem *)i;
-  if (col == 0)
-  {
-    if (getIndex() < item->getIndex())
-      return -1;
-    else if (getIndex() == item->getIndex())
-      return 0;
-    else
-      return 1;
-  }
-  else
-    return key(col, ascending).compare(i->key(col, ascending));
-}
-
 void Viewer::open()
 {
   QFileDialog *dialog = new QFileDialog(this);
@@ -344,32 +287,21 @@ void Viewer::open()
       else
       {
 	boost::scoped_ptr<QDialog> qdialog(new QDialog(this));
-	boost::scoped_ptr<Ui::KisenDialog> dialog(new Ui::KisenDialog());
+	boost::scoped_ptr<Ui_kisenDialog> dialog(new Ui_kisenDialog());
         dialog->setupUi(qdialog.get());
-	QTextCodec *codec = QTextCodec::codecForName("Shift-JIS");
-	for (size_t i = 0; i < ipx.size(); i++)
-	{
-	  std::string black = ipx.getPlayer(i, osl::BLACK);
-	  std::string white = ipx.getPlayer(i, osl::WHITE);
-	  std::string black_title = ipx.getTitle(i, osl::BLACK);
-	  std::string white_title = ipx.getTitle(i, osl::WHITE);
-	  new KisenItem(dialog->player,
-			i,
-			QString("%1").arg(i),
-			codec->toUnicode(black_title.c_str(),
-					 black_title.length()),
-			codec->toUnicode(black.c_str(), black.length()),
-			codec->toUnicode(white_title.c_str(),
-					 white_title.length()),
-			codec->toUnicode(white.c_str(), white.length()));
-	}
+        boost::scoped_ptr<KisenModel> kisen_model(new KisenModel(&ipx, this));
+        dialog->tableView->setModel(kisen_model.get());
+        dialog->tableView->verticalHeader()->hide();
+        dialog->tableView->resizeColumnsToContents();
+        dialog->tableView->setShowGrid(false);
 	int ret = qdialog->exec();
 	if (ret == QDialog::Accepted)
 	{
-	  KisenItem *item = (KisenItem *)dialog->player->selectedItem();
-	  if (item)
+          QModelIndexList selected =
+            dialog->tableView->selectionModel()->selection().indexes();
+	  if (!selected.empty())
 	  {
-	    open(fileName, item->getIndex());
+	    open(fileName, selected.first().row());
 	  }
 	}
       }
@@ -420,7 +352,6 @@ void Viewer::watchFile(bool enable)
 
 void Viewer::openUrl(bool reload)
 {
-#if QT_VERSION >= 0x040000
   HttpWindow *http = new HttpWindow(this, reload);
   http->show();
   if (http->exec() == QDialog::Accepted)
@@ -436,7 +367,6 @@ void Viewer::openUrl(bool reload)
       open(filename);
     kifuViewer->toLastState();
   }
-#endif
 }
 
 void Viewer::openUrl()
@@ -604,16 +534,6 @@ void Viewer::checkmateSimulation()
     showPage(checkmateViewer);
 }
 
-void Viewer::ntesukiSearch()
-{
-#ifdef ENABLE_NTESUKI
-  TabChild *child = (TabChild *)currentPage();
-  bool success = ntesukiViewer->analyze(child->getState());
-  if (success)
-    showPage(ntesukiViewer);
-#endif
-}
-
 void Viewer::quiescenceSearch()
 {
   TabChild *child = (TabChild *)currentPage();
@@ -634,14 +554,6 @@ void Viewer::quiescenceSearchHalf()
     showPage(quiescenceViewer);
 }
 
-void Viewer::piecePairDialog()
-{
-  TabChild *child = (TabChild *)currentPage();
-  PiecePairDialog *dialog = new PiecePairDialog(child->getState(), this);
-  dialog->show();
-  dialog->raise();
-}
-
 void Viewer::moveGenerateDialog()
 {
   if (currentPage()->inherits("BoardAndListTabChild"))
@@ -729,6 +641,28 @@ void Viewer::copyBoardAndMovesUsi()
   child->copyBoardAndMovesUsi();
 }
 
+void Viewer::paste()
+{
+  QWidget *widget = currentPage();
+  if (widget->inherits("KifuViewer"))
+  {
+    QClipboard *clipboard = QApplication::clipboard();
+    QString text = clipboard->text();
+    if (!text.isEmpty())
+    {
+      try
+      {
+        osl::record::csa::CsaString csa(text.toUtf8());
+        osl::stl::vector<osl::Move> board_moves;
+        ((KifuViewer *)widget)->open(csa.getInitialState(), board_moves);
+      }
+      catch(...)
+      {
+      }
+    }
+  }  
+}
+
 QString Viewer::getFilename()
 {
   TabChild *child = (TabChild *)currentPage();
@@ -821,6 +755,10 @@ void Viewer::highlightLastMove(bool on)
 {
   ((TabChild *)currentPage())->highlightLastMove(on);
 }
+void Viewer::highlightBookMove(bool on)
+{
+  ((TabChild *)currentPage())->highlightBookMove(on);
+}
 
 void Viewer::notifyEffect(QWidget *widget)
 {
@@ -838,14 +776,14 @@ void Viewer::kifuAnalyze()
 
 void Viewer::resetIcon()
 {
-#if QT_VERSION >= 0x040000 && (defined(__APPLE__) || defined(__FreeBSD__))
+#if defined(__APPLE__) || defined(__FreeBSD__)
   qApp->setWindowIcon(QIcon(":/images/icon.png"));
 #endif
 }
 
 void Viewer::setChatIcon()
 {
-#if QT_VERSION >= 0x040000 && (defined(__APPLE__) || defined(__FreeBSD__))
+#if defined(__APPLE__) || defined(__FreeBSD__)
   qApp->setWindowIcon(QIcon(":/images/icon-chat.png"));
 #endif
 }
@@ -875,26 +813,6 @@ void Viewer::editState()
   showPage(editor);
 }
 
-void Viewer::evalDebug()
-{
-  TabChild *child = (TabChild *)currentPage();
-  EvalDebugDialog *dialog = new EvalDebugDialog(child->getState(),
-						this);
-  if (child->inherits("BoardTabChild"))
-  {
-    BoardTabChild *board_tab_child = (BoardTabChild *)child;
-    connect(board_tab_child,
-	    SIGNAL(statusChanged(const osl::state::SimpleState &,
-				 const osl::stl::vector<osl::Move> &,
-				 int, osl::Move)),
-	    dialog, SLOT(setStatus(const osl::state::SimpleState &,
-				   const osl::stl::vector<osl::Move> &,
-				   int, osl::Move)));
-  }
-  dialog->show();
-  dialog->raise();
-}
-
 void Viewer::testEvalDebug()
 {
   TabChild *child = (TabChild *)currentPage();
@@ -915,6 +833,40 @@ void Viewer::testEvalDebug()
   dialog->raise();
 }
 
+void Viewer::setState1()
+{
+  TabChild *child = (TabChild *)currentPage();
+  state1.reset(new osl::state::SimpleState(child->getState()));
+}
+
+void Viewer::setState2()
+{
+  TabChild *child = (TabChild *)currentPage();
+  state2.reset(new osl::state::SimpleState(child->getState()));
+}
+
+void Viewer::showEvalDiff()
+{
+  if (state1.get() != NULL && state2.get() != NULL)
+  {
+    OpenMidEndingEvalDiffDialog *dialog =
+      new OpenMidEndingEvalDiffDialog(osl::state::NumEffectState(*state1),
+                                      osl::state::NumEffectState(*state2), this);
+    dialog->show();
+    dialog->raise();
+  }
+}
+
+void Viewer::showProgressDebug()
+{
+  TabChild *child = (TabChild *)currentPage();
+  osl::state::NumEffectState state(child->getState());
+  NewProgressDebugDialog *dialog = new NewProgressDebugDialog(state, this);
+  dialog->setAttribute(Qt::WA_DeleteOnClose);
+  dialog->show();
+  dialog->raise();
+}
+
 // ;;; Local Variables:
 // ;;; mode:c++
 // ;;; c-basic-offset:2
